"use strict";

var ISML = require("dw/template/ISML");
var Site = require("dw/system/Site");
var StringUtils = require("dw/util/StringUtils");
var allowedURLCharacters = /^[a-zA-Z0-9_%.?=/&+~!*()[\];:@,$#-]*$/g;
/**
 * It is used to block incoming requests and render error response returned from DataDome API.
 */
module.exports.Challenge = function () {
    var ddResponseHTML;
    if (session.privacy.ddRequestURL && session.privacy.ddResponseHTMLWithoutURL) {
        ddResponseHTML = String(session.privacy.ddResponseHTMLWithoutURL).replace(/'requestUrl'\s*:\s*'/, "'requestUrl':'" + session.privacy.ddRequestURL);
        session.privacy.ddResponseHTMLWithoutURL = null;
        session.privacy.ddRequestURL = null;
    } else {
        ddResponseHTML = session.privacy.ddResponseHTML;
        session.privacy.ddResponseHTML = null;
    }

    var queryString = request.getHttpQueryString();
    var match;
    if (queryString) {
        match = queryString.match("redirect=([^;&]+)");
    }

    var redirectUrl = match && match[1] ? match[1] : null;
    // get rid of unwanted characters
    redirectUrl = redirectUrl ? redirectUrl.replace(/(%40)|(%2e)|(%2e)|(%23)|°|(%E3%80%82)|(%0d)|(%0a)|(%0D)|(%07)|(%0b)|(%00)|(%09)|(%0D)|(%5C)|(%2F)/g, "") : null;
    redirectUrl = redirectUrl ? decodeURIComponent(redirectUrl) :  null;
    
    // avoid open redirect

    // redirect to current origin if it is an authentication
    if (redirectUrl && redirectUrl.match("@")) {
        redirectUrl = request.getHttpProtocol() + "://" + request.getHttpHost();
    }
    // redirect to host if the redirection url is not the main domain or listed in the params
    if (redirectUrl && (redirectUrl.toLowerCase().match(/^(https?:)?\/\//g)
    || redirectUrl.toLowerCase().match(/^(https?:)?\\\//g)
    || redirectUrl.toLowerCase().match(/^(https?:)?\\\\/g)
    || redirectUrl.toLowerCase().match(/^(https?:)?\/\\/g)
    || redirectUrl.toLowerCase().match(/^https?:/g))) {
        var ddRedirectOriginList = Site.getCurrent().getPreferences().custom.ddRedirectOriginList;
        var allowedOrigins = ddRedirectOriginList && ddRedirectOriginList.length > 0 ? ddRedirectOriginList.split(",") : [];
        allowedOrigins = allowedOrigins.map(function(item){ return item.trim(); });
        allowedOrigins.push(request.getHttpProtocol() + "://" + request.getHttpHost()); // allow current host in allowed list
        var reconstructedURL = request.getHttpProtocol() + ":" + redirectUrl;
        var isRedirectedToAllowedOrigin = allowedOrigins.some(function (v) { 
            return (redirectUrl.match(v) || reconstructedURL.match(v)); 
        });
        if (!isRedirectedToAllowedOrigin) {
            redirectUrl = request.getHttpProtocol() + "://" + request.getHttpHost(); // redirect to current origin
        }
    // redirect to current origin as we do not want to allow other absolute URLs
    } else if (redirectUrl && (redirectUrl.match(/:\/\//g)
    || redirectUrl.match(/:\\\\/g)
    || redirectUrl.match(/:\/\\/g)
    || redirectUrl.match(/:\\\//g))) { // to match other protocols like ftp://, ... //TODO: backslash
        redirectUrl = request.getHttpProtocol() + "://" + request.getHttpHost(); 
    }

    // convert unwanted characters from the URL to HTML entities
    if (redirectUrl && !redirectUrl.match(allowedURLCharacters)){
        redirectUrl = StringUtils.encodeString(redirectUrl, StringUtils.ENCODE_TYPE_HTML);
    }

    ISML.renderTemplate("error/ddresponse", {
        DDResponseHTML: ddResponseHTML,
        redirect: redirectUrl
    });
};


module.exports.Challenge.public = true;
