"use strict";

var HashMap = require("dw/util/HashMap");

/**
 * @module scripts/datadome/utils/mapHelper
 *
 * This script aims to improve Map functions
 */

/**
 * Builds a new object from an existing Map to allow case insensitive search 
 * @param {Map} existingMap 
 * @constructor
 */

function CaseInsensitiveMap(existingMap) {
    
    var internalMap = new HashMap();
    var keySet = existingMap.keySet();
  
    function normalizeKey(key) {
      if (typeof key === 'string') {
        return key.toLowerCase();
      } else {
        return key;
      }
    }

    for (var i = 0; i < keySet.size(); i++){
        internalMap.put(normalizeKey(keySet[i]), existingMap.get(keySet[i]));
    }
  
    this.put = function(key, value) {
      internalMap.put(normalizeKey(key), value);
    };
  
    this.get = function(key) {
      return internalMap.get(normalizeKey(key));
    };

    this.containsKey = function(key) {
      return internalMap.keySet().contains(normalizeKey(key));
    }

  }
    

  /* Module Exports */
exports.CaseInsensitiveMap = CaseInsensitiveMap;
